@@ -1,4 +1,16 @@
 /****************************************************************************/
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2006-2025 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
+/****************************************************************************/
 /// @file    RODFDetector.cpp
 /// @author  Daniel Krajzewicz
 /// @author  Eric Nicolay
@@ -8,31 +20,10 @@
 /// @author  Laura Bieker
 /// @author  Melanie Knocke
 /// @date    Thu, 16.03.2006
-/// @version $Id: RODFDetector.cpp 21851 2016-10-31 12:20:12Z behrisch $
 ///
 // Class representing a detector within the DFROUTER
 /****************************************************************************/
-// SUMO, Simulation of Urban MObility; see http://sumo.dlr.de/
-// Copyright (C) 2006-2016 DLR (http://www.dlr.de/) and contributors
-/****************************************************************************/
-//
-//   This file is part of SUMO.
-//   SUMO is free software: you can redistribute it and/or modify
-//   it under the terms of the GNU General Public License as published by
-//   the Free Software Foundation, either version 3 of the License, or
-//   (at your option) any later version.
-//
-/****************************************************************************/
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
-#ifdef _MSC_VER
-#include <windows_config.h>
-#else
 #include <config.h>
-#endif
 
 #include <cassert>
 #include "RODFDetector.h"
@@ -46,32 +37,28 @@
 #include "RODFRouteCont.h"
 #include "RODFDetectorFlow.h"
 #include <utils/vehicle/SUMOVTypeParameter.h>
-#include <utils/common/RandomDistributor.h>
+#include <utils/distribution/RandomDistributor.h>
 #include <utils/common/StdDefs.h>
-#include <utils/common/TplConvert.h>
+#include <utils/common/StringUtils.h>
 #include <utils/geom/GeomHelper.h>
 #include "RODFNet.h"
 #include <utils/iodevices/OutputDevice.h>
 #include <utils/common/StringUtils.h>
 #include <utils/options/OptionsCont.h>
 
-#ifdef CHECK_MEMORY_LEAKS
-#include <foreign/nvwa/debug_new.h>
-#endif // CHECK_MEMORY_LEAKS
-
 
 // ===========================================================================
 // method definitions
 // ===========================================================================
 RODFDetector::RODFDetector(const std::string& id, const std::string& laneID,
-                           SUMOReal pos, const RODFDetectorType type)
-    : Named(id), myLaneID(laneID), myPosition(pos), myType(type), myRoutes(0) {}
+                           double pos, const RODFDetectorType type)
+    : Named(id), myLaneID(laneID), myPosition(pos), myType(type), myRoutes(nullptr) {}
 
 
 RODFDetector::RODFDetector(const std::string& id, const RODFDetector& f)
     : Named(id), myLaneID(f.myLaneID), myPosition(f.myPosition),
-      myType(f.myType), myRoutes(0) {
-    if (f.myRoutes != 0) {
+      myType(f.myType), myRoutes(nullptr) {
+    if (f.myRoutes != nullptr) {
         myRoutes = new RODFRouteCont(*(f.myRoutes));
     }
 }
@@ -88,10 +75,10 @@ RODFDetector::setType(RODFDetectorType type) {
 }
 
 
-SUMOReal
+double
 RODFDetector::computeDistanceFactor(const RODFRouteDesc& rd) const {
-    SUMOReal distance = rd.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(rd.edges2Pass.back()->getToJunction()->getPosition());
-    SUMOReal length = 0;
+    double distance = rd.edges2Pass[0]->getFromJunction()->getPosition().distanceTo(rd.edges2Pass.back()->getToJunction()->getPosition());
+    double length = 0;
     for (ROEdgeVector::const_iterator i = rd.edges2Pass.begin(); i != rd.edges2Pass.end(); ++i) {
         length += (*i)->getLength();
     }
@@ -103,7 +90,7 @@ void
 RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCon& detectors,
                                         const RODFDetectorFlows& flows,
                                         SUMOTime startTime, SUMOTime endTime, SUMOTime stepOffset) {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         return;
     }
     // compute edges to determine split probabilities
@@ -129,7 +116,7 @@ RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCo
             }
         }
     }
-    std::map<ROEdge*, SUMOReal> inFlows;
+    std::map<ROEdge*, double> inFlows;
     if (OptionsCont::getOptions().getBool("respect-concurrent-inflows")) {
         for (std::vector<RODFEdge*>::const_iterator i = nextDetEdges.begin(); i != nextDetEdges.end(); ++i) {
             std::set<ROEdge*> seen(preSplitEdges);
@@ -156,11 +143,11 @@ RODFDetector::computeSplitProbabilities(const RODFNet* net, const RODFDetectorCo
     // compute the probabilities to use a certain direction
     int index = 0;
     for (SUMOTime time = startTime; time < endTime; time += stepOffset, ++index) {
-        mySplitProbabilities.push_back(std::map<RODFEdge*, SUMOReal>());
-        SUMOReal overallProb = 0;
+        mySplitProbabilities.push_back(std::map<RODFEdge*, double>());
+        double overallProb = 0;
         // retrieve the probabilities
         for (std::vector<RODFEdge*>::const_iterator i = nextDetEdges.begin(); i != nextDetEdges.end(); ++i) {
-            SUMOReal flow = detectors.getAggFlowFor(*i, time, 60, flows) - inFlows[*i];
+            double flow = detectors.getAggFlowFor(*i, time, 60, flows) - inFlows[*i];
             overallProb += flow;
             mySplitProbabilities[index][*i] = flow;
         }
@@ -179,9 +166,9 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
         SUMOTime startTime, SUMOTime endTime, SUMOTime stepOffset,
         const RODFNet& net,
         std::map<SUMOTime, RandomDistributor<int>* >& into) const {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         if (myType != DISCARDED_DETECTOR && myType != BETWEEN_DETECTOR) {
-            WRITE_ERROR("Missing routes for detector '" + myID + "'.");
+            WRITE_ERRORF(TL("Missing routes for detector '%'."), myID);
         }
         return;
     }
@@ -189,39 +176,39 @@ RODFDetector::buildDestinationDistribution(const RODFDetectorCon& detectors,
     // iterate through time (in output interval steps)
     for (SUMOTime time = startTime; time < endTime; time += stepOffset) {
         into[time] = new RandomDistributor<int>();
-        std::map<ROEdge*, SUMOReal> flowMap;
+        std::map<ROEdge*, double> flowMap;
         // iterate through the routes
         int index = 0;
         for (std::vector<RODFRouteDesc>::iterator ri = descs.begin(); ri != descs.end(); ++ri, index++) {
-            SUMOReal prob = 1.;
+            double prob = 1.;
             for (ROEdgeVector::iterator j = (*ri).edges2Pass.begin(); j != (*ri).edges2Pass.end() && prob > 0;) {
                 if (!net.hasDetector(*j)) {
                     ++j;
                     continue;
                 }
                 const RODFDetector& det = detectors.getAnyDetectorForEdge(static_cast<RODFEdge*>(*j));
-                const std::vector<std::map<RODFEdge*, SUMOReal> >& probs = det.getSplitProbabilities();
+                const std::vector<std::map<RODFEdge*, double> >& probs = det.getSplitProbabilities();
                 if (probs.size() == 0) {
                     prob = 0;
                     ++j;
                     continue;
                 }
-                const std::map<RODFEdge*, SUMOReal>& tprobs = probs[(time - startTime) / stepOffset];
-                RODFEdge* splitEdge = 0;
-                for (std::map<RODFEdge*, SUMOReal>::const_iterator k = tprobs.begin(); k != tprobs.end(); ++k) {
+                const std::map<RODFEdge*, double>& tprobs = probs[(int)((time - startTime) / stepOffset)];
+                RODFEdge* splitEdge = nullptr;
+                for (std::map<RODFEdge*, double>::const_iterator k = tprobs.begin(); k != tprobs.end(); ++k) {
                     if (find(j, (*ri).edges2Pass.end(), (*k).first) != (*ri).edges2Pass.end()) {
                         prob *= (*k).second;
                         splitEdge = (*k).first;
                         break;
                     }
                 }
-                if (splitEdge != 0) {
-                    j = find(j, (*ri).edges2Pass.end(), splitEdge);
+                if (splitEdge != nullptr) {
+                    j = std::find(j, (*ri).edges2Pass.end(), splitEdge);
                 } else {
                     ++j;
                 }
             }
-            into[time]->add(prob, index);
+            into[time]->add(index, prob);
             (*ri).overallProb = prob;
         }
     }
@@ -268,7 +255,7 @@ RODFDetector::addRoutes(RODFRouteCont* routes) {
 
 void
 RODFDetector::addRoute(RODFRouteDesc& nrd) {
-    if (myRoutes == 0) {
+    if (myRoutes == nullptr) {
         myRoutes = new RODFRouteCont();
     }
     myRoutes->addRouteDesc(nrd);
@@ -277,7 +264,7 @@ RODFDetector::addRoute(RODFRouteDesc& nrd) {
 
 bool
 RODFDetector::hasRoutes() const {
-    return myRoutes != 0 && myRoutes->get().size() != 0;
+    return myRoutes != nullptr && myRoutes->get().size() != 0;
 }
 
 
@@ -288,16 +275,16 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                                      SUMOTime startTime, SUMOTime endTime,
                                      SUMOTime stepOffset,
                                      bool includeUnusedRoutes,
-                                     SUMOReal scale,
+                                     double scale,
                                      bool insertionsOnly,
-                                     SUMOReal defaultSpeed) const {
+                                     double defaultSpeed) const {
     OutputDevice& out = OutputDevice::getDevice(file);
     OptionsCont& oc = OptionsCont::getOptions();
     if (getType() != SOURCE_DETECTOR) {
-        out.writeXMLHeader("calibrator");
+        out.writeXMLHeader("additional", "additional_file.xsd");
     }
     // routes
-    if (myRoutes != 0 && myRoutes->get().size() != 0) {
+    if (myRoutes != nullptr && myRoutes->get().size() != 0) {
         const std::vector<RODFRouteDesc>& routes = myRoutes->get();
         out.openTag(SUMO_TAG_ROUTE_DISTRIBUTION).writeAttr(SUMO_ATTR_ID, myID);
         bool isEmptyDist = true;
@@ -307,19 +294,19 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
             }
         }
         for (std::vector<RODFRouteDesc>::const_iterator i = routes.begin(); i != routes.end(); ++i) {
-            if ((*i).overallProb > 0 || includeUnusedRoutes) {
-                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, (*i).overallProb).closeTag();
-            }
             if (isEmptyDist) {
-                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, SUMOReal(1)).closeTag();
+                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, 1.0).closeTag();
+            } else if ((*i).overallProb > 0 || includeUnusedRoutes) {
+                out.openTag(SUMO_TAG_ROUTE).writeAttr(SUMO_ATTR_REFID, (*i).routename).writeAttr(SUMO_ATTR_PROB, (*i).overallProb).closeTag();
             }
         }
         out.closeTag(); // routeDistribution
     } else {
-        WRITE_ERROR("Detector '" + getID() + "' has no routes!?");
+        WRITE_ERRORF(TL("Detector '%' has no routes!?"), getID());
         return false;
     }
     // insertions
+    int vehicleIndex = 0;
     if (insertionsOnly || flows.knows(myID)) {
         // get the flows for this detector
         const std::vector<FlowDef>& mflows = flows.getFlowDefs(myID);
@@ -332,14 +319,36 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
             // get flows at end
             RandomDistributor<int>* destDist = dists.find(time) != dists.end() ? dists.find(time)->second : 0;
             // go through the cars
-            int carNo = (int)((srcFD.qPKW + srcFD.qLKW) * scale);
-            for (int car = 0; car < carNo; ++car) {
+            const int numCars = (int)((srcFD.qPKW + srcFD.qLKW) * scale);
+
+
+            std::vector<SUMOTime> departures;
+            if (oc.getBool("randomize-flows")) {
+                for (int i = 0; i < numCars; ++i) {
+                    departures.push_back(time + RandHelper::rand(stepOffset));
+                }
+                std::sort(departures.begin(), departures.end());
+            } else {
+                for (int i = 0; i < numCars; ++i) {
+                    departures.push_back(time + (stepOffset * i / numCars));
+                }
+            }
+
+            for (int car = 0; car < numCars; ++car) {
                 // get the vehicle parameter
-                SUMOReal v = -1;
+                double v = -1;
                 std::string vtype;
-                int destIndex = destDist != 0 && destDist->getOverallProb() > 0 ? (int) destDist->get() : -1;
+                int destIndex = -1;
+                if (destDist != nullptr) {
+                    if (destDist->getOverallProb() > 0) {
+                        destIndex = destDist->get();
+                    } else if (myRoutes->get().size() > 0) {
+                        // equal probabilities. see writeEmitterDefinition()
+                        destIndex = RandHelper::rand((int)myRoutes->get().size());
+                    }
+                }
                 if (srcFD.isLKW >= 1) {
-                    srcFD.isLKW = srcFD.isLKW - (SUMOReal) 1.;
+                    srcFD.isLKW = srcFD.isLKW - 1.;
                     v = srcFD.vLKW;
                     vtype = "LKW";
                 } else {
@@ -350,18 +359,14 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 if (v <= 0 || v > 250) {
                     v = defaultSpeed;
                 } else {
-                    v = (SUMOReal)(v / 3.6);
+                    v /= 3.6;
                 }
                 // compute the departure time
-                SUMOTime ctime = (SUMOTime)(time + ((SUMOReal) stepOffset * (SUMOReal) car / (SUMOReal) carNo));
+                const SUMOTime ctime = departures[car];
 
                 // write
                 out.openTag(SUMO_TAG_VEHICLE);
-                if (getType() == SOURCE_DETECTOR) {
-                    out.writeAttr(SUMO_ATTR_ID, "emitter_" + myID + "_" + toString(ctime));
-                } else {
-                    out.writeAttr(SUMO_ATTR_ID, "calibrator_" + myID + "_" + toString(ctime));
-                }
+                out.writeAttr(SUMO_ATTR_ID, myID + "." + toString(vehicleIndex));
                 if (oc.getBool("vtype")) {
                     out.writeAttr(SUMO_ATTR_TYPE, vtype);
                 }
@@ -369,19 +374,19 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 if (oc.isSet("departlane")) {
                     out.writeNonEmptyAttr(SUMO_ATTR_DEPARTLANE, oc.getString("departlane"));
                 } else {
-                    out.writeAttr(SUMO_ATTR_DEPARTLANE, TplConvert::_2int(myLaneID.substr(myLaneID.rfind("_") + 1).c_str()));
+                    out.writeAttr(SUMO_ATTR_DEPARTLANE, SUMOXMLDefinitions::getIndexFromLane(myLaneID));
                 }
                 if (oc.isSet("departpos")) {
                     std::string posDesc = oc.getString("departpos");
                     if (posDesc.substr(0, 8) == "detector") {
-                        SUMOReal position = myPosition;
+                        double position = myPosition;
                         if (posDesc.length() > 8) {
                             if (posDesc[8] == '+') {
-                                position += TplConvert::_2SUMOReal(posDesc.substr(9).c_str());
+                                position += StringUtils::toDouble(posDesc.substr(9));
                             } else if (posDesc[8] == '-') {
-                                position -= TplConvert::_2SUMOReal(posDesc.substr(9).c_str());
+                                position -= StringUtils::toDouble(posDesc.substr(9));
                             } else {
-                                throw NumberFormatException();
+                                throw NumberFormatException("");
                             }
                         }
                         out.writeAttr(SUMO_ATTR_DEPARTPOS, position);
@@ -412,6 +417,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
                 }
                 out.closeTag();
                 srcFD.isLKW += srcFD.fLKW;
+                vehicleIndex++;
             }
         }
     }
@@ -425,7 +431,7 @@ RODFDetector::writeEmitterDefinition(const std::string& file,
 bool
 RODFDetector::writeRoutes(std::vector<std::string>& saved,
                           OutputDevice& out) {
-    if (myRoutes != 0) {
+    if (myRoutes != nullptr) {
         return myRoutes->save(saved, "", out);
     }
     return false;
@@ -436,19 +442,19 @@ void
 RODFDetector::writeSingleSpeedTrigger(const std::string& file,
                                       const RODFDetectorFlows& flows,
                                       SUMOTime startTime, SUMOTime endTime,
-                                      SUMOTime stepOffset, SUMOReal defaultSpeed) {
+                                      SUMOTime stepOffset, double defaultSpeed) {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("vss");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     const std::vector<FlowDef>& mflows = flows.getFlowDefs(myID);
     int index = 0;
     for (SUMOTime t = startTime; t < endTime; t += stepOffset, index++) {
         assert(index < (int)mflows.size());
         const FlowDef& srcFD = mflows[index];
-        SUMOReal speed = MAX2(srcFD.vLKW, srcFD.vPKW);
+        double speed = MAX2(srcFD.vLKW, srcFD.vPKW);
         if (speed <= 0 || speed > 250) {
             speed = defaultSpeed;
         } else {
-            speed = (SUMOReal)(speed / 3.6);
+            speed = (double)(speed / 3.6);
         }
         out.openTag(SUMO_TAG_STEP).writeAttr(SUMO_ATTR_TIME, time2string(t)).writeAttr(SUMO_ATTR_SPEED, speed).closeTag();
     }
@@ -481,7 +487,7 @@ RODFDetectorCon::addDetector(RODFDetector* dfd) {
     }
     myDetectorMap[dfd->getID()] = dfd;
     myDetectors.push_back(dfd);
-    std::string edgeid = dfd->getLaneID().substr(0, dfd->getLaneID().rfind('_'));
+    const std::string edgeid = SUMOXMLDefinitions::getEdgeIDFromLane(dfd->getLaneID());
     if (myDetectorEdgeMap.find(edgeid) == myDetectorEdgeMap.end()) {
         myDetectorEdgeMap[edgeid] = std::vector<RODFDetector*>();
     }
@@ -521,7 +527,7 @@ RODFDetectorCon::getDetectors() const {
 void
 RODFDetectorCon::save(const std::string& file) const {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("detectors");
+    out.writeXMLHeader("detectors", "detectors_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         out.openTag(SUMO_TAG_DETECTOR_DEFINITION).writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML((*i)->getID())).writeAttr(SUMO_ATTR_LANE, (*i)->getLaneID()).writeAttr(SUMO_ATTR_POSITION, (*i)->getPos());
         switch ((*i)->getType()) {
@@ -549,7 +555,7 @@ RODFDetectorCon::save(const std::string& file) const {
 void
 RODFDetectorCon::saveAsPOIs(const std::string& file) const {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("pois");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         out.openTag(SUMO_TAG_POI).writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML((*i)->getID()));
         switch ((*i)->getType()) {
@@ -577,7 +583,7 @@ RODFDetectorCon::saveAsPOIs(const std::string& file) const {
 void
 RODFDetectorCon::saveRoutes(const std::string& file) const {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("routes");
+    out.writeXMLHeader("routes", "routes_file.xsd");
     std::vector<std::string> saved;
     // write for source detectors
     bool lastWasSaved = true;
@@ -621,7 +627,7 @@ RODFDetectorCon::writeEmitters(const std::string& file,
                                SUMOTime stepOffset, const RODFNet& net,
                                bool writeCalibrators,
                                bool includeUnusedRoutes,
-                               SUMOReal scale,
+                               double scale,
                                bool insertionsOnly) {
     // compute turn probabilities at detector
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
@@ -629,30 +635,30 @@ RODFDetectorCon::writeEmitters(const std::string& file,
     }
     //
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("additional", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/additional_file.xsd\"");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     // write vType(s)
     const bool separateVTypeOutput = OptionsCont::getOptions().getString("vtype-output") != "";
     OutputDevice& vTypeOut = separateVTypeOutput ? OutputDevice::getDevice(OptionsCont::getOptions().getString("vtype-output")) : out;
     if (separateVTypeOutput) {
-        vTypeOut.writeXMLHeader("additional", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/additional_file.xsd\"");
+        vTypeOut.writeXMLHeader("additional", "additional_file.xsd");
     }
     const bool forceDev = !OptionsCont::getOptions().isDefault("speeddev");
-    const SUMOReal speedDev = OptionsCont::getOptions().getFloat("speeddev");
+    const double speedDev = OptionsCont::getOptions().getFloat("speeddev");
     if (OptionsCont::getOptions().getBool("vtype")) {
         // write separate types
         SUMOVTypeParameter pkwType = SUMOVTypeParameter("PKW", SVC_PASSENGER);
         setSpeedFactorAndDev(pkwType, net.getMaxSpeedFactorPKW(), net.getAvgSpeedFactorPKW(), speedDev, forceDev);
-        pkwType.setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+        pkwType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
         pkwType.write(vTypeOut);
         SUMOVTypeParameter lkwType = SUMOVTypeParameter("LKW", SVC_TRUCK);
         setSpeedFactorAndDev(lkwType, net.getMaxSpeedFactorLKW(), net.getAvgSpeedFactorLKW(), speedDev, forceDev);
-        lkwType.setParameter |= VTYPEPARS_VEHICLECLASS_SET;
+        lkwType.parametersSet |= VTYPEPARS_VEHICLECLASS_SET;
         lkwType.write(vTypeOut);
     } else {
         // patch default type
         SUMOVTypeParameter type = SUMOVTypeParameter(DEFAULT_VTYPE_ID, SVC_PASSENGER);
         setSpeedFactorAndDev(type, MAX2(net.getMaxSpeedFactorPKW(), net.getMaxSpeedFactorLKW()), net.getAvgSpeedFactorPKW(), speedDev, forceDev);
-        if (type.setParameter != 0) {
+        if (type.parametersSet != 0) {
             type.write(vTypeOut);
         }
     }
@@ -672,7 +678,7 @@ RODFDetectorCon::writeEmitters(const std::string& file,
             continue;
         }
         // try to write the definition
-        SUMOReal defaultSpeed = net.getEdge(det->getEdgeID())->getSpeed();
+        double defaultSpeed = net.getEdge(det->getEdgeID())->getSpeedLimit();
         //  ... compute routes' distribution over time
         std::map<SUMOTime, RandomDistributor<int>* > dists;
         if (!insertionsOnly && flows.knows(det->getID())) {
@@ -698,18 +704,20 @@ RODFDetectorCon::writeEmitters(const std::string& file,
 }
 
 void
-RODFDetectorCon::setSpeedFactorAndDev(SUMOVTypeParameter& type, SUMOReal maxFactor, SUMOReal avgFactor, SUMOReal dev, bool forceDev) {
+RODFDetectorCon::setSpeedFactorAndDev(SUMOVTypeParameter& type, double maxFactor, double avgFactor, double dev, bool forceDev) {
     if (avgFactor > 1) {
         // systematically low speeds can easily be caused by traffic
         // conditions. Whereas elevated speeds probably reflect speeding
-        type.speedFactor = avgFactor;
-        type.setParameter |= VTYPEPARS_SPEEDFACTOR_SET;
+        type.speedFactor.setParameter(0, avgFactor);
+        type.parametersSet |= VTYPEPARS_SPEEDFACTOR_SET;
     }
-    if (forceDev || (maxFactor > 1 && maxFactor > type.speedFactor + NUMERICAL_EPS)) {
+    if (forceDev || (maxFactor > 1 && maxFactor > type.speedFactor.getParameter(0) + NUMERICAL_EPS)) {
         // setting a non-zero speed deviation causes the simulation to recompute
         // individual speedFactors to match departSpeed (MSEdge::insertVehicle())
-        type.speedDev = dev;
-        type.setParameter |= VTYPEPARS_SPEEDDEVIATION_SET;
+        type.speedFactor.setParameter(1, dev);
+        type.parametersSet |= VTYPEPARS_SPEEDFACTOR_SET;
+    } else {
+        type.speedFactor.setParameter(1, -1.); // do not write speedDev, only simple speedFactor
     }
 }
 
@@ -718,10 +726,10 @@ void
 RODFDetectorCon::writeEmitterPOIs(const std::string& file,
                                   const RODFDetectorFlows& flows) {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("additional", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/additional_file.xsd\"");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         RODFDetector* det = *i;
-        SUMOReal flow = flows.getFlowSumSecure(det->getID());
+        double flow = flows.getFlowSumSecure(det->getID());
         const unsigned char col = static_cast<unsigned char>(128 * flow / flows.getMaxDetectorFlow() + 128);
         out.openTag(SUMO_TAG_POI).writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML((*i)->getID()) + ":" + toString(flow));
         switch ((*i)->getType()) {
@@ -751,15 +759,15 @@ RODFDetectorCon::getAggFlowFor(const ROEdge* edge, SUMOTime time, SUMOTime perio
                                const RODFDetectorFlows&) const {
     UNUSED_PARAMETER(period);
     UNUSED_PARAMETER(time);
-    if (edge == 0) {
+    if (edge == nullptr) {
         return 0;
     }
-//    SUMOReal stepOffset = 60; // !!!
-//    SUMOReal startTime = 0; // !!!
+//    double stepOffset = 60; // !!!
+//    double startTime = 0; // !!!
 //    cout << edge->getID() << endl;
     assert(myDetectorEdgeMap.find(edge->getID()) != myDetectorEdgeMap.end());
     const std::vector<FlowDef>& flows = static_cast<const RODFEdge*>(edge)->getFlows();
-    SUMOReal agg = 0;
+    double agg = 0;
     for (std::vector<FlowDef>::const_iterator i = flows.begin(); i != flows.end(); ++i) {
         const FlowDef& srcFD = *i;
         if (srcFD.qLKW >= 0) {
@@ -772,7 +780,7 @@ RODFDetectorCon::getAggFlowFor(const ROEdge* edge, SUMOTime time, SUMOTime perio
     return (int) agg;
     /* !!! make this time variable
     if (flows.size()!=0) {
-        SUMOReal agg = 0;
+        double agg = 0;
         int beginIndex = (int)((time/stepOffset) - startTime);  // !!! falsch!!!
         for (SUMOTime t=0; t<period&&beginIndex<flows.size(); t+=(SUMOTime) stepOffset) {
             const FlowDef &srcFD = flows[beginIndex++];
@@ -797,14 +805,14 @@ RODFDetectorCon::writeSpeedTrigger(const RODFNet* const net,
                                    SUMOTime startTime, SUMOTime endTime,
                                    SUMOTime stepOffset) {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("additional", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/additional_file.xsd\"");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         RODFDetector* det = *i;
         // write the declaration into the file
         if (det->getType() == SINK_DETECTOR && flows.knows(det->getID())) {
             std::string filename = FileHelpers::getFilePath(file) + "vss_" + det->getID() + ".def.xml";
             out.openTag(SUMO_TAG_VSS).writeAttr(SUMO_ATTR_ID, StringUtils::escapeXML(det->getID())).writeAttr(SUMO_ATTR_LANES, det->getLaneID()).writeAttr(SUMO_ATTR_FILE, filename).closeTag();
-            SUMOReal defaultSpeed = net != 0 ? net->getEdge(det->getEdgeID())->getSpeed() : (SUMOReal) 200.;
+            double defaultSpeed = net != nullptr ? net->getEdge(det->getEdgeID())->getSpeedLimit() : (double) 200.;
             det->writeSingleSpeedTrigger(filename, flows, startTime, endTime, stepOffset, defaultSpeed);
         }
     }
@@ -815,13 +823,13 @@ RODFDetectorCon::writeSpeedTrigger(const RODFNet* const net,
 void
 RODFDetectorCon::writeEndRerouterDetectors(const std::string& file) {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("additional", "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://sumo.dlr.de/xsd/additional_file.xsd\"");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         RODFDetector* det = *i;
         // write the declaration into the file
         if (det->getType() == SINK_DETECTOR) {
             out.openTag(SUMO_TAG_REROUTER).writeAttr(SUMO_ATTR_ID, "endrerouter_" + StringUtils::escapeXML(det->getID())).writeAttr(SUMO_ATTR_EDGES, det->getLaneID());
-            out.writeAttr(SUMO_ATTR_POSITION, SUMOReal(0)).writeAttr(SUMO_ATTR_FILE, "endrerouter_" + det->getID() + ".def.xml").closeTag();
+            out.writeAttr(SUMO_ATTR_POSITION, 0.).writeAttr(SUMO_ATTR_FILE, "endrerouter_" + det->getID() + ".def.xml").closeTag();
         }
     }
     out.close();
@@ -833,17 +841,17 @@ RODFDetectorCon::writeValidationDetectors(const std::string& file,
         bool includeSources,
         bool singleFile, bool friendly) {
     OutputDevice& out = OutputDevice::getDevice(file);
-    out.writeXMLHeader("additional");
+    out.writeXMLHeader("additional", "additional_file.xsd");
     for (std::vector<RODFDetector*>::const_iterator i = myDetectors.begin(); i != myDetectors.end(); ++i) {
         RODFDetector* det = *i;
         // write the declaration into the file
         if (det->getType() != SOURCE_DETECTOR || includeSources) {
-            SUMOReal pos = det->getPos();
+            double pos = det->getPos();
             if (det->getType() == SOURCE_DETECTOR) {
                 pos += 1;
             }
             out.openTag(SUMO_TAG_E1DETECTOR).writeAttr(SUMO_ATTR_ID, "validation_" + StringUtils::escapeXML(det->getID())).writeAttr(SUMO_ATTR_LANE, det->getLaneID());
-            out.writeAttr(SUMO_ATTR_POSITION, pos).writeAttr(SUMO_ATTR_FREQUENCY, 60);
+            out.writeAttr(SUMO_ATTR_POSITION, pos).writeAttr(SUMO_ATTR_PERIOD, 60);
             if (friendly) {
                 out.writeAttr(SUMO_ATTR_FRIENDLY_POS, true);
             }
@@ -867,7 +875,7 @@ RODFDetectorCon::removeDetector(const std::string& id) {
     myDetectorMap.erase(ri1);
     //
     std::vector<RODFDetector*>::iterator ri2 =
-        find(myDetectors.begin(), myDetectors.end(), oldDet);
+        std::find(myDetectors.begin(), myDetectors.end(), oldDet);
     myDetectors.erase(ri2);
     //
     bool found = false;
@@ -897,7 +905,7 @@ RODFDetectorCon::guessEmptyFlows(RODFDetectorFlows& flows) {
         const std::set<const RODFDetector*>& follower = det->getFollowerDetectors();
         int noFollowerWithRoutes = 0;
         int noPriorWithRoutes = 0;
-        // count occurences of detectors with/without routes
+        // count occurrences of detectors with/without routes
         std::set<const RODFDetector*>::const_iterator j;
         for (j = prior.begin(); j != prior.end(); ++j) {
             if (flows.knows((*j)->getID())) {
@@ -945,7 +953,7 @@ RODFDetectorCon::getAnyDetectorForEdge(const RODFEdge* const edge) const {
 void
 RODFDetectorCon::clearDists(std::map<SUMOTime, RandomDistributor<int>* >& dists) const {
     for (std::map<SUMOTime, RandomDistributor<int>* >::iterator i = dists.begin(); i != dists.end(); ++i) {
-        delete(*i).second;
+        delete (*i).second;
     }
 }
 
@@ -964,4 +972,9 @@ RODFDetectorCon::mesoJoin(const std::string& nid,
 }
 
 
+std::string
+RODFDetector::getEdgeID() const {
+    return SUMOXMLDefinitions::getEdgeIDFromLane(myLaneID);
+}
+
 /****************************************************************************/
