@@ -1,11 +1,15 @@
 /****************************************************************************/
-// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.org/sumo
-// Copyright (C) 2002-2019 German Aerospace Center (DLR) and others.
-// This program and the accompanying materials
-// are made available under the terms of the Eclipse Public License v2.0
-// which accompanies this distribution, and is available at
-// http://www.eclipse.org/legal/epl-v20.html
-// SPDX-License-Identifier: EPL-2.0
+// Eclipse SUMO, Simulation of Urban MObility; see https://eclipse.dev/sumo
+// Copyright (C) 2002-2025 German Aerospace Center (DLR) and others.
+// This program and the accompanying materials are made available under the
+// terms of the Eclipse Public License 2.0 which is available at
+// https://www.eclipse.org/legal/epl-2.0/
+// This Source Code may also be made available under the following Secondary
+// Licenses when the conditions for such availability set forth in the Eclipse
+// Public License 2.0 are satisfied: GNU General Public License, version 2
+// or later which is available at
+// https://www.gnu.org/licenses/old-licenses/gpl-2.0-standalone.html
+// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0-or-later
 /****************************************************************************/
 /// @file    RORoute.h
 /// @author  Daniel Krajzewicz
@@ -16,13 +20,7 @@
 ///
 // A complete router's route
 /****************************************************************************/
-#ifndef RORoute_h
-#define RORoute_h
-
-
-// ===========================================================================
-// included modules
-// ===========================================================================
+#pragma once
 #include <config.h>
 
 #include <string>
@@ -65,7 +63,7 @@ public:
      */
     RORoute(const std::string& id, double costs, double prob,
             const ConstROEdgeVector& route, const RGBColor* const color,
-            const std::vector<SUMOVehicleParameter::Stop>& stops);
+            const StopParVector& stops);
 
 
     /** @brief Constructor
@@ -168,9 +166,15 @@ public:
      */
     void recheckForLoops(const ConstROEdgeVector& mandatory);
 
+
+    /// @brief check whether the route is valid for the given vehicle
+    bool isValid(const ROVehicle& veh, bool ignoreErrors) const;
+
     OutputDevice&
     writeXMLDefinition(OutputDevice& dev, const ROVehicle* const veh,
-                       const bool withCosts, const bool withExitTimes) const;
+                       const bool withCosts, const bool asAlternatives,
+                       const bool withExitTimes, const bool withLength,
+                       const std::string& id = "") const;
 
     /** @brief add additional vehicles/probability
      */
@@ -180,20 +184,23 @@ public:
      *
      * @return list of stops
      */
-    const std::vector<SUMOVehicleParameter::Stop>& getStops() const {
+    const StopParVector& getStops() const {
         return myStops;
     }
 
     /** @brief Adapts the until time of all stops by the given offset
      */
     void addStopOffset(const SUMOTime offset) {
-        for (std::vector<SUMOVehicleParameter::Stop>::iterator stop = myStops.begin(); stop != myStops.end(); ++stop) {
+        for (StopParVector::iterator stop = myStops.begin(); stop != myStops.end(); ++stop) {
             if (stop->until >= 0) {
                 stop->until += offset;
             }
         }
     }
 
+    /// @brief return edges that shall be written in the route definition
+    ConstROEdgeVector getNormalEdges() const;
+
 private:
     /// @brief The costs of the route
     double myCosts;
@@ -208,7 +215,7 @@ private:
     const RGBColor* myColor;
 
     /// @brief List of the stops on the parsed route
-    std::vector<SUMOVehicleParameter::Stop> myStops;
+    StopParVector myStops;
 
 
 private:
@@ -216,9 +223,3 @@ private:
     RORoute& operator=(const RORoute& src);
 
 };
-
-
-#endif
-
-/****************************************************************************/
-
